;+
;Purpose: A crib showing how to create 2-D slices through Themis particle
;         distributions using thm_part_dist_array, thm_part_slice2d, 
;         and plot_part_slice2d
;         
;         Run "thm_ui_slice2d" on the IDL console to use for the GUI version.
;         (Also part of the Analysis menu on the main THEMIS GUI) 
;
;
;NOTES:  Thm_part_slice2d can create three types of slices:
;   2D Nearest Neighbor (default):
;     A slice of data with user defined width is projected onto the slice plane 
;     and interpolated onto a regular grid using the nearest neighbor method.  
;
;   2D Interpolation (from thm_esa_slice2d):
;     Datapoints within the specified theta or z-axis range are projected onto 
;     the slice plane and linearly interpolated onto a regular 2d grid.
;
;   3D Interpolation:
;     The entire 3-dimensional distribution is linearly interpolated onto a 
;     regular 3d grid and a this slice is extracted.  The slice will consist of
;     up to resolution^2 datapoints from within 0.866/resolution of the total 
;     velocity range along the normal. This method works best with dense data.
;       Note: - Not all points within the aforemention range will be used.
;             - Interpolation may occure across data gaps or areas with recorded zeroes
;             - Higher resolution regridding will severely slow this method
;
;    *More detailed documentation for slices can be found the THEMIS User's guide and in:
 ;      thm_part_slice2d.pro
;       plot_part_slice2d.pro
;       thm_part_dist_array.pro
;      
;
;ROTATIONS: Available rotations (based off native coords, e.g. dsl, gse):
;   'BV': The x axis is parallel to B field; the bulk velocity defines the x-y plane. (DEFAULT)
;   'BE': The x axis is parallel to B field; the B x V(bulk) vector defines the x-y plane.
;   'xy'/'xyz': The x axis is V_x and the y axis is V_y.
;   'xz': The x axis is V_x and the y axis is V_z.
;   'yz': The x axis is V_y and the y axis is V_z.
;   'xvel': The x axis is V_x; the y axis is defined by the bulk velocity. 
;   'perp': The x axis is the bulk velocity projected onto the plane normal to the B field; y is B x V(bulk)
;   'perp_xy'/'perp_xyz': Geometric xy coordinates projected onto the plane normal to the B field.
;   'perp_xz': Geometric xz coordinates projected onto the plane normal to the B field.
;   'perp_yz': Geometric yz coordinates projected onto the plane normal to the B field.
;
;UNITS: Available units (DF by default):
;        'Counts','DF','Rate','CRate','Flux','EFlux','E2Flux','E3Flux')
;
;CAVEATS: Due to IDL software constraints areas of no data are assigned zeros
;         instead of NaNs. This yields a degeneracy between data gaps and 
;         measured zeros for both plots and numerical data.
;         (Such degeneracy would always exist in any logarithmic plot made
;         with the contour procedure)
;
;-

PRO thm_crib_part_slice2d
;===========================================================
; Load Data
;===========================================================

; Set time range of interest to create array of particle distributions
day = '2008-02-26/'
start_time = time_double(day + '04:52:00')
end_time = time_double(day + '04:52:30')

; Pad time range to ensure enough data is loaded
trange = [start_time - 120, end_time + 120]

probe = 'b'
format = 'thb_peib' ;type of particle data to load

; Load magnetic field and level 2 velocity data (for coord rotation).
; Mag data is required for all coordinates other than geometric (xyz).  
; Velocity data is required for BV, BE, perp, and xvel.
; If velocity data is not loaded explicitly it can be automatically 
; calculated from the distribution (may be less accurate).
thm_load_fgm, probe=probe, datatype = 'fgh', level=2, coord='dsl', trange=trange
thm_load_esa, probe=probe, datatype = 'peib_velocity_dsl', trange=trange

; Create array of ESA particle distributions using the L2 velocities
peib_dist_arr = thm_part_dist_array(format=format,  $
                           ;you could use the line below instead of the FORMAT keyword
                           ;type='peib', probe='b', $
                           trange=trange, $
                           mag_data='thb_fgh_dsl', $
                           vel_data='thb_peib_velocity_dsl')
                           ;you could use the libe below instead of the VEL_DATA keyword
                           ;vel_auto=1)



;===========================================================
; Create Slice
;===========================================================

;type = 1  ; Set this optional keyword to change how the slice is produced:
          ; 1 - 2D Nearest neighbor interpolation (default) 
          ; 2 - 2D Linear Interpolation (same as thm_esa_slice2d, fewer options)
          ; 3 - 3D Linear Interpolation (caveats listed above)

; Time
;-----
slice_time = time_double(day + '04:52:00') ; time at which to calc slice
timewin=30.    ; set the time window to 30 seconds
;center_time=1  ; set this keyword to make SLICE_TIME the center of the time window


; Units
;------
;units = 'counts' ; set this keyword to change the units used ('DF' is default)


; Coordinates and Rotation
;-------------------------
coord='gsm' ;slice based off GSM coordinates
;coord='gse' ;slice based off GSE coordinates

;Rotations are performed in the requested coordinates (DSL unless otherwise specified)
;See documentation at begining of the crib for more options.
;rotation='xyz' ; no rotation, uses specified coordinates 
rotation='BV' ; the x axis is parallel to B field; the bulk velocity defines the x-y plane.

;(2D nearest neighbor and 3D interpolation only)
;Cuts are made along the x-y axis by default, the following keyword specifies 
;the slice plane's normal and and offset from zero along that normal (in km/s)
slice_orient=[0, 0, 1, 0] ; slice plane is the x-y axis
;slice_orient=[1, 0, 0, 400] ; slice plane is the y-z axis translated 400 km/s from the origin


; Other Options
;--------------
;By default, data is first regridded to 32 x 16 x 128 points in phi, theta and energy respectively 
;regrid = [64,32,64]  ; set this keyword to change the dimensions of the regridding
;regrid = 0           ; set to 0 to turn off regridding 

;(2D nearest neighbor only)
;Set this to change the width of the slice (in km/s); if not set 5% of the data range 
;will be used by default. Increase width for better resolution near plot edges.
;Decrease for fewer "blotches" at lower velocities.   
slice_width = 10   ; slice width of 10% the entire range

resolution=100 ; set the number of elements in each dimension of the slice

;Smooth the slice
;smooth = 3 ; set this keyword to an odd integer >=3 specifying the width of the smoothing window


; Create the slice
;-----------------
thm_part_slice2d, peib_dist_arr, slice_time=slice_time, timewin=timewin, $
                  center_time=center_time, slice_orient=slice_orient, $
                  resolution=resolution, rotation=rotation, type = type, $
                  smooth=smooth, coord=coord, slice_width=slice_width, $
                  part_slice=part_slice, $ ; slice data
                  xgrid=xgrid, $           ; x values
                  ygrid=ygrid, $           ; y values
                  slice_info=slice_info, $ ; plotting info
                  fail=fail

if keyword_set(fail) then print, fail


;===========================================================
; Plot Slice
;===========================================================

; Set keywords for plotting
nlines = 60 ;number of color levels
olines = 8 ;number of contour line levels
;range = [1.E-14,1.E-6]  ; limit data range for plotting
;xrange = [1000.,-1000]  ; specify xrange to plot
;yrange = xrange         ; specify yrange to plot

; Plot
plot_part_slice2d, part_slice, $
                   xgrid, $
                   ygrid, $
                   slice_info, $
;                   range=range, $
;                   xrange=xrange, $
;                   yrange=yrange, $
;                   sundir = 1b, $ ;plot sun direction (may load STATE data)
                   nlines=nlines, $
                   olines=olines
END




;===========================================================
; Multiple Slices
;===========================================================
;+
; 
; A Secondary Crib: Create Multiple Slices and Export:
; 
; 
;-

PRO thm_crib_part_slice2d_multi

; Choose output location:

outputfolder='C:\THEMIS\slice_crib\test2\' ;Modify and uncomment this line if you're using Windows
;outputfolder='/THEMIS/slice_crib/'  ;Modify and uncomment this line if you're using Mac/Unix/Linux



; Load Data
;-----------

; Set time range
day = '2008-02-26/'
start_time = time_double(day + '04:50:00')
end_time = time_double(day + '04:55:00')

; pad time range to ensure enough data is loaded
trange=[start_time - 90, end_time + 90]

; set data types
probe = 'b'
format = 'thb_pseb'

; This example will use geometric coordinates so skip loading of mag & velocity data
;thm_load_fgm, probe=probe, datatype = 'fgh', level=2, coord='dsl', trange=trange
;thm_load_esa, probe=probe, datatype = 'peeb_velocity_dsl', trange=trange

; Create array of SST particle distributions
dist_arr = thm_part_dist_array(format=format,  $
                       ;you could use the line below instead of the FORMAT keyword
;                       type='pseb', probe='b', $
                       trange=trange, $
;                       mag_data='thb_fgh_dsl', $
;                       vel_data='thb_peeb_velocity_dsl')
                       vel_auto=1)



;Set Options
;------------

timewin = 30.   ; set the time window for each slice
incriment = 15. ; time incriment for next slice's start

coord = 'gsm'   ; GSM coordinates

rotation='xyz'  ; see description at beginning of crib

slice_orient = [0,1,0,0] ; slice along x-z plane

;units = 'counts'     ; 'DF' by default

regrid = [64,32,64] ; regrid options for phi,theta,energy ([32,16,128] by default)
;regrid = 0          ; disable regridding

;smooth = 3    ; smooth data w/ boxcar average (3 point window size) 

;resolution = 100  ; slice resolution

slice_width = 5 ; slice width in %

range = [2.2e-27, 2.2e-20] ; plot using fixed range this time

nlines = 60
olines = 8



; Use loop to create multiple slices and export plots
;----------------------------------------------------

export_to = 'png'  ; export to .png images
;export_to = 'eps'  ; export to postscript

slice_time = start_time

while slice_time lt end_time do begin
  
  ;Create slice
  thm_part_slice2d, dist_arr, slice_time=slice_time, timewin=timewin, $
                    center_time=center_time, slice_orient=slice_orient, $
                    resolution=resolution, rotation=rotation, coord=coord, $
                    smooth=smooth, regrid=regrid, slice_width=slice_width, $
                    part_slice=part_slice, $
                    xgrid=xgrid, $
                    ygrid=ygrid, $
                    slice_info=slice_info, $
                    fail=fail

  ; the FAIL variable will contain a string message if something goes wrong
  if keyword_set(fail) then begin
    print, 'An error occured while creating the slice at '+time_string(slice_time)+':'
    print, fail
  endif

  ;create filename for image
  clock_time = time_string(format=2,slice_time)
  outputfile = outputfolder + clock_time + format

  ;open file if exporting to postscript
  if export_to eq 'eps' then begin
    popen, outputfile, encapsulated = 1
  endif

  ;Call plotting procedure (NOTE: Multiple slices can be created and stored before plotting)
  plot_part_slice2d, part_slice, $
                   xgrid, $
                   ygrid, $
                   slice_info, $
                   range=range, $ ;constant range this time
                   nlines=nlines, $
                   olines=olines

  ;close postscript file
  if export_to eq 'eps' then begin
    pclose
  endif

  ;write current plot to a .png in outputfolder
  if export_to eq 'png' then begin
    makepng, outputfile, /mkdir
  endif

  slice_time += incriment ;increment time
  
endwhile


END

