
;+
;NAME:
; thm_sst_calib_params2
;PURPOSE:
;  This populates the correct fields of the dist struct so that data can be correctly calibrated.
;
;Inputs:
;  dist: The dist struct that will have its fields populated
;  param_structs: An array of parameter structs that contains the calibration parameters for each time
;
;Outputs:
;  error: Set to 1 if error occurred and 0 otherwise.  If error occurs calling routine should halt.
;    
;NOTES:
;  #1 dist.time should be set by the time this is called
;  #2 uses the calibration parameters for full distribution on all types.
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-12-16 17:26:18 -0800 (Thu, 16 Dec 2010) $
;$LastChangedRevision: 7929 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/spacecraft/particles/SST/SST_cal_workdir/thm_sst_calib_params2.pro $
;-


pro thm_sst_calib_params2,dist,param_structs,error=err

  err=1

  thm_init
  
  num_params=87
    
  tg = tag_names(param_structs[0])
  
  if n_elements(tg) ne num_params then begin
    dprint,'ERROR: Incorrect number of parameters in calibration file.',dlevel=0
    dprint,'CAL File is incorrectly formatted or you need to download updated code',dlevel=0
    dprint,'Distribution Type: ' + dist.tplotname,dlevel=0
    dist = 0
    return 
  endif

  ;determine the correct parameters to use according to sample time
  ;uses the nearest parameters to dist.time that are dated before dist.time
  t = dist.time - time_double((param_structs[*]).time)

  idx = where(t gt 0,c) 
  
  if c eq 0 then begin
    dprint,'ERROR: calibration file has no parameters for this time',dlevel=0
    dprint,'CAL File is incorrectly formatted or you need to download updated code',dlevel=0
    dist = 0
    return 
  endif
  
  temp = min(t[idx],idx2)
  param_struct = param_structs[idx[idx2]]
  
  ;using n to denote the start of this param type in the cal file
  ;makes code more flexible when adding new parameters or moving files around
  n = 1
  
  for i = 0,3 do begin
  ;scaling factors to convert DAP bin numbers into eV
  ;allows different values for each theta(telescope aperture)
    dist.energy_scales[i] = param_struct.(n+i)
  endfor
  n += 4
  
  for i = 0,3 do begin
  ;offsets to convert scaled DAP bin numbers into eV
  ;allows different values for each theta(telescope aperture)
    dist.energy_offsets[i] = param_struct.(n+i) 
  endfor
  
  n += 4
  
  for i = 0,3 do begin
  ;
  ;allows different values for each theta(telescope aperture)
    dist.energy_reciprocals[i] = param_struct.(n+i) 
  endfor
  
  n+= 4
  
  ;dead time correction factor(applies one value for all telescopes)
  dist.dead = param_struct.(n)
  
  n++
  
  ;nominal(theoretical) geometric factor
  dist.geom_factor = param_struct.(n)
    
  n++
    
  det_idx = lindgen(16)
  
  ;Temporary. Following calibration parameters are only valid atm for full & burst modes
  dim = dimen(dist.data)
  if n_elements(dim) eq 1 || dim[1] ne 64 then begin
    err=0
    return
  endif
  
  for i = 0,3 do begin 
    ;correction factors to nominal geometric factor.  Allows for variance in manufacturing, and mechanical tolerances.
    ;allows different values for each theta(telescope aperture)
    ;note that values are rebinned so that one value is repeated for all energies/phi.  This way data can be scaled element by element.
    dist.gf[*,det_idx+i*16] = param_struct.(n+i)
  endfor
  
  n += 4
  
  for i = 0,3 do begin
    ;attenuator scaling factors.  Vary from the nominal(1/64), due to manufacturing and mechnical tolerances.  
    ;allows different values for each theta(telescope aperture)
    ;note that values are rebinned so that one value is repeated for all energies/phi.  This way data can be scaled element by element.
    dist.att[*,det_idx+i*16] = param_struct.(n+i)
  endfor 
  
  n += 4
  
  for i = 0,3 do begin
    for j = 0,15 do begin
      ;detector PHA bin efficiencies. 
      ;different values are stored for each combination of theta and energy, but will be the same across phi
       ;note that values are rebinned so that one value is repeated for all phis.  This way data can be scaled element by element.
      dist.eff[j,det_idx+i*16] = param_struct.(i*16+j+n)
    endfor
  endfor

  err = 0

end