
; Helper function,
; Calculates cartesian velocities based off angle and energy bins.
; Sphereically regrids (interpolates) data if requested.
;
pro thm_part_slice2d_getXYZ, thedata, in, regrid=regrid, $
                           validbins=validbins, currbins=currbins, $
                           erange=erange, q=q, conv=conv, $
                           ncounts = ncounts, $
                           xvels = xvels, $
                           yvels = yvels, $
                           zvels = zvels, $
                           orange=orange, $
                           fail=fail, skip=skip

  compile_opt idl2, hidden

  skip = 0b

  ; constants
  mass = thedata.mass / conv ; conversion: (mass ~= proton | electron mass)
  if keyword_set(regrid) then begin
    if n_elements(regrid) lt 3 then begin
      fail = 'REGRID keyword must contain 3 elements [phi,theta,energy]'
      return
    endif
  
    method = 'NearestNeighbor'
    phigrid = interpol( [0.,360], regrid[0])
    thetagrid = interpol( minmax(thedata.theta), regrid[1])
    
    enum = round(regrid[2]/thedata.nenergy)
    seed_tmp = (findgen(enum)-(0.5)*(enum-1)) / enum
    dprint, 'Interpolating each energy to '+strtrim(enum,2)+' energies.'
  endif

  
  ; Find active, valid bins in specified erange.  The valid/not valid
  ; status will be stored in boolean arrays: 'currbins' for the current 
  ; distribution and 'validbins' for the overall data set. 
  currbins = (thedata.bins ne 0 $
              and thedata.energy le erange[1] $
              and thedata.energy ge erange[0] $
              and finite(thedata.data) eq 1)
  valid = where(currbins, nvalid)
  if nvalid le 0 then begin
    skip = 1b
    return
  endif


  ; Determine and pass out extrema of original data for plotting
  notzero = where(thedata.data[valid] ne 0, nnotzero)
  if nnotzero gt 0 then begin
    mm = minmax( (thedata.data[valid])[notzero] )
    if ~keyword_set(orange) then orange = mm
    orange = [ (orange[0] lt mm[0] ? orange[0]:mm[0]), $
               (orange[1] gt mm[1] ? orange[1]:mm[1])  ]
  endif


  ; Convert to cartesian coordinates
  ;

  ; Regrid data in spherical coords if requested
  ;---------------------------------------------
  if keyword_set(regrid) then begin
    
    for i=0, thedata.nenergy-1 do begin

      ; Find valid bins for each energy
      loopbins = where(currbins[i,*], nlb)
      if nlb le 0 then continue
      
      ; Angles and data must be copied into new variable for triangulation/gridding
      phitemp = reform(thedata.phi[i,loopbins])
      thetatemp = reform(thedata.theta[i,loopbins])
      tempcounts = reform(thedata.data[i,loopbins])
      
      if regrid[0] lt n_elements(phitemp[uniq(phitemp, sort(phitemp))]) or $
         regrid[1] lt n_elements(thetatemp[uniq(thetatemp, sort(thetatemp))]) then begin
         dprint, "WARNING: Current regrid settings are below the current data's resolution."
      endif
      
      ; Create a (required) sphereical triangulation
      qhull, phitemp, thetatemp, triangles, sphere=dummy
      
      ; Interpolate this energy's data onto a regular spherical grid
      newgrid = griddata(phitemp, thetatemp, tempcounts, $
                         method=method, /sphere, /degrees, $
;                         min_points=9, $
;                         max_per_sector=12, $
                         triangles=triangles, $
                         /grid, xout=phigrid, yout=thetagrid)

      ; Replicate data if interpolating across energies
      if keyword_set(enum) then begin
        newgrid = rebin(newgrid, [size(newgrid,/dim),enum], /sample)
      endif

      ; Concatenate data arrays, needs to have single dimension
      tempgrid = ~keyword_set(tempgrid) ? reform(newgrid,n_elements(newgrid)): $
                                 [tempgrid, reform(newgrid,n_elements(newgrid))]

  
      ; Only calculate cartesian velocity components once.
      ; Changes in angle bins are checked elsewhere.
      if in eq 0 then begin
      
        ;energy value should not be changing across the selected bins (checked above)
        if keyword_set(enum) then begin
          denergyGrid = (seed_tmp * median(thedata.denergy[i,loopbins])) + median(thedata.energy[i,loopbins])
          energyvalue = sqrt(2*q*(denergyGrid[0])/mass)
        endif else begin
          energyvalue = sqrt(2*q*median(thedata.energy[i,loopbins])/mass)
        endelse
  
        ; x & y velocity arrays
        x = cos(!pi/180*phigrid) # cos(!pi/180*thetagrid) * energyvalue
        y = sin(!pi/180*phigrid) # cos(!pi/180*thetagrid) * energyvalue
  
        ; z velocity array corresponding to x & y arrays
        ztmp = sin(!pi/180*thetagrid) *energyvalue
        z = rebin(transpose(ztmp), size(x,/dim), /sample)
;        z2 = fltarr(size(x,/dim),/nozero) ;todo: remove
;        for j=0, n_elements(ztmp)-1 do replicate_inplace, z2, ztmp[j], 1, [0,j]
;        if ~array_equal(z, z2) then stop
  
        ; concatenate to arrays from previous loops
        first = ~keyword_set(xvels)
        xvels = first ? reform(x,n_elements(x)):[xvels, reform(x,n_elements(x))]
        yvels = first ? reform(y,n_elements(y)):[yvels, reform(y,n_elements(y))]
        zvels = first ? reform(z,n_elements(z)):[zvels, reform(z,n_elements(z))]

        ; add velocities corresponding interpolated energies
        if keyword_set(enum) then begin
          factorGrid = sqrt(denergyGrid / denergyGrid[0])
          for l=1, enum-1 do begin ; 0th was already done above
            xvels = [temporary(xvels), reform(factorGrid[l] * x,n_elements(x))]
            yvels = [temporary(yvels), reform(factorGrid[l] * y,n_elements(y))]
            zvels = [temporary(zvels), reform(factorGrid[l] * z,n_elements(z))]
          endfor
        endif
        
      endif
      
    endfor

    ; Pass out interpolated data
    ncounts = temporary(tempgrid)

    ; All bins are valid in this case
    currbins = replicate(1b, n_elements(ncounts))
    validbins = currbins


  ;If not regriding convert directely...
  ;-----------------------------------
  endif else begin
  
    if in eq 0 then begin

      ; Calculate cartesian velocity componenets of each bin for entire distribution
      ; datapoints outside requested limits will be removed before slice creation.
      sphere_to_cart, sqrt(2*q*thedata.energy/mass), thedata.theta, thedata.phi, $
                      xvels, yvels , zvels
  
    endif

    ; Array of counts for each nrg/angle bin...same size as xvels
    ncounts = reform(thedata.data, n_elements(thedata.data))

    ; Modify boolean array describing entire data set
    if ~keyword_set(validbins) then validbins = currbins
    validbins = currbins or validbins

    ; Zero invalid interies to prevent them being added to the valid entries.
    ;  (datapoints w/ no valid entries will be removed later)
    tozero = where(~currbins, ntozero)
    if ntozero gt 0 then ncounts[tozero] = 0

  endelse

  Return

end



;Check for changes in angle bins
;
function thm_part_slice2d_checkbins, thedata, datstructs, fail=fail

    compile_opt idl2, hidden

  diff = ''
  if in_set(thedata.theta eq datstructs[0].theta,0) then diff += 'THETA '
  if in_set(thedata.phi eq datstructs[0].phi,0) then diff += 'THETA '
  if in_set(thedata.energy eq datstructs[0].energy,0) then diff += 'ENERGY '
  if diff ne '' then begin
    diff = strjoin(strsplit(diff,/extract),', ')
    fail = diff+' values differ between distributions at '+ $
           time_string(thedata.time)+' and '+time_string(datstructs[0].time)
    dprint, fail
    return, 0
  endif

  return, 1

end



; Returns vector with mag data or -1 on error
;
function thm_part_slice2d_getmag, thedata, datstructs, magdata=magdata, $
                                rotation=rotation, fail=fail

    compile_opt idl2, hidden
  
  if keyword_set(magdata) then begin
    ; average mag data over time window
    bfield = dat_avg(magdata, datStructs[0].time, thedata.end_time)
  endif else begin
    ; average mag data over time window
    dprint, 'Using bfield data from dat structure.'
    bfield = average(datStructs.magf, 2)
    
    ; check for NaNs that could corrupt rotation matrix
    if in_set(finite(bfield),0) then begin
      if ~in_set(['xy','xz','yz','xvel'], rotation) then begin
        fail = 'Magnetic field data incomplete for desired time range.'
        dprint, fail
        return, -1
      endif
    endif 
    
  endelse
  
    return, bfield

end



;Returns bulk velocity vector or -1 on error
;
;Note: thm_esa_slice2d Divides total flux (over entire time range) by 
;   the total density; here the flux/density was previously 
;   calculated for each distribution and is now averaged.
function thm_part_slice2d_getvel, thedata, datstructs, veldata=veldata, $
                                rotation=rotation, fail=fail

    compile_opt idl2, hidden
  
  if keyword_set(veldata) then begin
    ;user specified velocitys can currently be added to the 
    ;dat structure in thm_part_dist_array 
    dprint, 'Using specified velocity: '+veldata
    thevel = dat_avg(veldata, datStructs[0].time, thedata.end_time)
  endif else begin
  
    dprint, 'Using velocity tag from dat structure.'
    thevel = average(datStructs.velocity, 2) ;vel in km/s
    
    ; Check for NaNs that could corrupt rotation matrix
    if in_set(finite(thevel), 0) then begin
      if in_set(['BV','BE','xvel','perp'], rotation) then begin 
        fail='Bulk velocity data incomplete for desired time range.'+ $
             '  Unable to perform corrdinate rotation to '+rotation+'.'
        dprint, fail
        return, -1
      endif
    endif
  
  endelse

  ; convert to m/s
  return, thevel * 1000.

end


;Performs rotations from DSL
; *Assumes the transformation from the data's native DSL to
;  the requested coords (GSM, GSE) is invariant over the
;  time range of the slice.* 
pro thm_part_slice2d_cotrans, vectors, thevel, bfield, $
                              probe=probe,coord=coord, $
                              twin=twin, ct=ct, $
                              fail=fail

    compile_opt idl2, hidden


  ctn = '2dslice_temp_cotrans'
  if stregex(coord, '^dsl$', /bool, /fold_case) then return
  if ~keyword_set(probe) then begin
    dprint, 'Cannot determine spacecraft. Coordinate transform canceled.'
    coord = 'DSL'
    return
  endif
  if ~stregex(coord, '(^gsm$)|(^gse$)', /bool, /fold_case) then begin
    fail = 'Invalid coordinate keyword: '+coord
    dprint, fail
    return
  endif
  probe = strlowcase(probe)
  coord = strlowcase(coord)


  ;Load necessary support data (2 min padding at each end)
  thm_load_state, probe=probe, trange = twin + 120*[-1,1], suffix='_'+ctn, $
                  /get_support_data


  ;Transform x,y,z unit vectors to new coordinates
  store_data, ctn, data = {x:replicate(mean(twin),3), $
                           y: [ [1.,0,0], [0,1,0], [0,0,1] ] }
  thm_cotrans, ctn, probe=probe, in_coord='dsl', out_coord=coord, $
               support_suffix='_'+ctn, out_suff='_'+coord


  ;Form transformation matrix from transformed basis
  get_data, ctn+'_'+coord, data=ctd
  if size(ctd,/type) ne 8 then begin
    fail = 'Could not obtain coordinate transform from DSL -> '+strupcase(coord)+ $
           ', STATE data may be absent for the requested period.'
    dprint, fail
    return
  endif
  ct = transpose(ctd.y)


  ;Apply to velocity data and required rotation vectors
  vectors = ct ## temporary(vectors)
  if keyword_set(thevel) then thevel = ct ## thevel
  if keyword_set(bfield) then bfield = ct ## bfield


  ;Delete temporary tplot data
  store_data, '*'+ctn+'*', /delete

end


;Perform rotation on cartesian velocity bins
pro thm_part_slice2d_rotate, velocities, rot, bfield, thevel, rotation=rotation, fail=fail

    compile_opt idl2, hidden

  ; Create rotation matrix
  case rotation of
    'BV': rot=cal_rot(bfield,thevel)
    'BE': rot=cal_rot(bfield,crossp(bfield,thevel))
    'xy': rot=cal_rot([1,0,0],[0,1,0])
    'xz': rot=cal_rot([1,0,0],[0,0,1])
    'yz': rot=cal_rot([0,1,0],[0,0,1])
    'xvel': rot=cal_rot([1,0,0],thevel)
    'perp': rot=cal_rot(crossp(crossp(bfield,thevel),bfield),crossp(bfield,thevel))
    'perp_yz': rot=cal_rot(crossp(crossp(bfield,[0,1,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    'perp_xy': rot=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,1,0]),bfield))
    'perp_xz': rot=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    else: begin
      fail = 'Unrecognized rotation: "'+rotation+'".'
      dprint, fail
      return
    end
  endcase
  
  ; Prevent data from being mutated to doubles
  rot = float(rot)
  
  ; Rotate data
  velocities = rot ## temporary(velocities)
  if keyword_set(thevel) then thevel = rot ## thevel
  if keyword_set(bfield) then bfield = rot ## bfield

end


; Produces the slice's normal and x-vector
;
pro thm_part_slice2d_orientslice, slice_orient, fail=fail, $
                                  orient = orient, $
                                  displacement = displacement, $
                                  norm = unit_norm, $
                                  xvec = xvec

    compile_opt idl2, hidden

  ; Create unit vector for slice orientation
  slice_norm = float(slice_orient[0:2])
  displacement = float(slice_orient[3])
  unit_norm = slice_norm / sqrt(total(slice_norm^2)) ; convert slice normal to a unit vec
  if in_set(finite(unit_norm), 0) then begin
    fail = 'Invalid orientation vector.'
    return
  endif
  
  ; Project current x axis into slice plane or use y axis if orthogonal
  a = crossp(unit_norm,crossp([1,0,0], unit_norm))
  if total(abs(a)) eq 0. then begin
    if unit_norm[0] lt 0 then xvec=[0, -1, 0] else xvec=[0, 1, 0]
  endif else begin
    xvec = a  / sqrt(total(a^2))
  endelse

  orient = cal_rot(xvec, crossp(unit_norm,xvec))

end


; Subtracts bulk velocity vector from velocities array
;
pro thm_part_slice2d_subtract, velocities, thevel, veldata

    compile_opt idl2, hidden

  if keyword_set(veldata) then begin
    dprint,'Velocity used for subtraction is '+veldata
  endif else begin
    dprint, 'Velocity used for subtraction is V_3D'
  endelse

  velocities[*,0] = velocities[*,0] - thevel[0]
  velocities[*,1] = velocities[*,1] - thevel[1]
  velocities[*,2] = velocities[*,2] - thevel[2]

end



; Set slicing limits for 2d interpolation method (from thm_esa_slice2d)
;
pro thm_part_slice2d_2dlimits, velocities, datapoints, $
                               thetarange=thetarange, zdirrange=zdirrange, $
                               fail = fail

    compile_opt idl2, hidden

  ; Cut by theta value
  if keyword_set(thetarange) then begin
  
    thetarange = minmax(thetarange)
    
    r = sqrt(velocities[*,0]^2 + velocities[*,1]^2 + velocities[*,2]^2)
    eachangle = asin(velocities[*,2]/r)/!dtor
    
    index = where(eachangle le thetarange[1] and eachangle ge thetarange[0], $
                  count, ncomplement=ncount)
    if count ne 0 then begin
      if ncount ne 0 then begin
        velocities = velocities[index,*]
        datapoints = datapoints[index]
        print, 'Angle (theta) = '+strtrim(thetarange,2)
      endif
    endif else begin
      fail = 'No data points in given theta range'
      dprint, fail
      return
    endelse
    
  ;Cut by z-axis value
  endif else if keyword_set(zdirrange) then begin
    
    zdirrange = minmax(zdirrange)
    
    index = where(velocities[*,2] ge zdirrange[0] and velocities[*,2] le zdirrange[1], $
                  count, ncomplement=ncount)
    if count ne 0 then begin
      if ncount ne 0 then begin
        velocities = velocities[index,*]
        datapoints = datapoints[index]
        print, 'Z-Direction range = '+strtrim(zdirrange,2) 
      endif
    endif else begin
      fail = 'No data points in given Z direction range'
      dprint, fail
      return
    endelse  
  endif

end



; Produces slice using 2D linear interpolation
;
pro thm_part_slice2d_type2, datapoints, velocities, resolution, $
                            part_slice=part_slice, $
                            xgrid=xgrid, ygrid=ygrid, $
                            fail=fail

    compile_opt idl2, hidden


  xvels = velocities[*,0]
  yvels = temporary(velocities[*,1])

  ; Average duplicate points ---------
  uni2=uniq(xvels)
  uni1=[0,uni2[0:n_elements(uni2)-2]+1]
  
  kk=0
  for i=0,n_elements(uni2)-1 do begin
      yvelsi=yvels[uni1[i]:uni2[i]]
      xvelsi=xvels[uni1[i]:uni2[i]]
      datapointsi=datapoints[uni1[i]:uni2[i]]
  
      xvelsi=xvelsi[sort(yvelsi)]
      datapointsi=datapointsi[sort(yvelsi)]
      yvelsi=yvelsi[sort(yvelsi)]
  
    index2=uniq(yvelsi)
    if n_elements(index2) eq 1 then begin
        index1=0
    endif else begin
        index1=[0,index2[0:n_elements(index2)-2]+1]
    endelse
  
      for j=0,n_elements(index2)-1 do begin
          yvels[kk]=yvelsi[index1[j]]
          xvels[kk]=xvelsi[index1[j]]
          if index1[j] eq index2[j] then begin
              datapoints[kk]=datapointsi[index1[j]]
          endif else begin
              datapoints_moment=moment(datapointsi[index1[j]:index2[j]])
              datapoints[kk]=datapoints_moment[0]
          endelse
          kk=kk+1
      endfor
  endfor

  yvels=yvels[0:kk-1]
  xvels=xvels[0:kk-1]
  datapoints=datapoints[0:kk-1]
  ; ---------------


  ; Create triangulation
  triangulate,xvels,yvels,tr,b
  
  ; Remove triangles whose total x-y plane velocity is less than
  ; minimum velocity from distribution (prevents interpolation over
  ; lower energy limits)
  index = where( 1./9 * total( xvels[tr[0:2,*]] ,1 )^2 + $
                 1./9 * total( yvels[tr[0:2,*]] ,1 )^2 $
                  gt min(xvels^2+yvels^2), $
                  count, ncomplement=ncomp)
  if count gt 0 then begin
    if ncomp gt 0 then begin
      tr=tr[*,index]
    endif
  endif else dprint, 'Possible error in triangulation' 
  
  ; Set spacing
  xmax = max(abs([yvels,xvels]))
  xrange = [-1*xmax,xmax]
  spacing = (xrange[1]-xrange[0])/(resolution-1)
  
  ; Interpolate to regular grid (slice data)
  part_slice = trigrid(xvels,yvels,datapoints,tr,[spacing,spacing], $
                       [xrange[0],xrange[0],xrange[1],xrange[1]],   $
                       xgrid = xgrid,ygrid = ygrid )


end



; Produces slice from linearly interpolated 3D grid
;
pro thm_part_slice2d_type3, datapoints, velocities, resolution, $
                            slice_orient, orange, $
                            part_slice=part_slice, $
                            xgrid=xgrid, ygrid=ygrid, $
                            fail=fail

    compile_opt idl2, hidden


  ; Get slice's orienation vectors
  thm_part_slice2d_orientslice, slice_orient, fail=fail, $
                                displacement = displacement, $
                                norm = norm, xvec = xvec
  if keyword_set(fail) then return


  ; Create cube grid
  velmm = [ [minmax(velocities[*,0])], [minmax(velocities[*,1])], [minmax(velocities[*,1])] ]
  xgrid = interpol(velmm[*,0], resolution)
  ygrid = interpol(velmm[*,1], resolution)
  zgrid = interpol(velmm[*,2], resolution)

  ; Get slice's center point
  center = ( (norm * displacement)/(velmm[1,*]-velmm[0,*]) + 0.5) * resolution
  if in_set(center le 0 or center gt resolution, 1) then begin
    fail = 'Slice displacement is outside the data range.'
    return
  endif

  ; Must be copied to new variables for qhull
  xvels = velocities[*,0]
  yvels = velocities[*,1]
  zvels = temporary(velocities[*,2])
  
  qhull, xvels, yvels, zvels, th, /DELAUNAY
  
  ; Remove tetrahedra whose total velocity (centroid) is less than
  ; minimum velocity from distribution (prevents interpolation over
  ; lower energy limits)
  index = where( 1./16 * total(  xvels[th[0:3,*]] ,1 )^2 + $
                 1./16 * total(  yvels[th[0:3,*]] ,1 )^2 + $
                 1./16 * total(  zvels[th[0:3,*]] ,1 )^2  $
                  gt min(xvels^2+yvels^2+zvels^2), $
                  count, ncomplement=ncomp)
  if count gt 0 then begin
    if ncomp gt 0 then begin
      th=th[*,index]
    endif
  endif else dprint, 'Possible error in triangulation' 

  ; Interpolate data to regular 3D grid
  vol = qgrid3(xvels, yvels, zvels, datapoints, th, dimension=replicate(resolution,3))
  
  ; Remove erroneous data points (also helps prevent interpolation over gaps)
  derp = where(abs(vol) lt orange[0] or abs(vol) gt orange[1], nderp)
  if nderp gt 0 then begin
    vol[derp] = 0
  endif 

  ; Extract slice from regular grid
  part_slice=extract_slice(vol, resolution, resolution, $
                           center[0], center[1], center[2], $
                           norm, xvec, /sample)

;  ; For testing only: shows 3-D distribution of non-zero data (using valid scaling)
;  ww = where(vol gt 0, nww)
;  tmparr = bytscl(alog10(vol[ww]))
;  x_idxs = where(   vol[ww] eq min( (vol[ww])[where(vol[ww] ge orange[0])] )   )
;  x = float( tmparr[x_idxs[0]] )
;  m = -255./(x-255)
;  c = (255.*x)/(x-255)
;  tmparr = m*tmparr + c
;  ltz = where(tmparr lt 0, nltz)
;  if nltz gt 0 then tmparr[ltz] = 0
;  ;tmparr[where(tmparr gt 255)] = 255
;  tmparr = byte(round(tmparr))
;  x_idx = ww mod n_elements(xgrid) 
;  y_idx = (ww / n_elements(xgrid)) mod n_elements(ygrid)
;  z_idx = ww / (n_elements(xgrid)*n_elements(ygrid))
;  rg2 = [-max(xgrid),max(xgrid)]
;  iplot, xgrid[x_idx], ygrid[y_idx], zgrid[z_idx], linestyle=6, sym_index=1, $
;         xrange = rg2, yrange=rg2, zrange=rg2, $
;         rgb_table=13, vert_colors=tmparr


;  ; For testing only: View volume in IDL medical imaging software
;  mm = minmax(vol, min_value=0)
;  vol= temporary(vol)/mm[0]
;  vol = alog10(temporary(vol))
;  mm = minmax(vol, min_value=0)
;  vol = BYTSCL(temporary(vol), min=mm[0], max=mm[1])
;  hData = PTR_NEW(vol) ; slicer3 needs a pointer to the volume data 
;  slicer3, hData

end



; Produces 2D nearest neighbor slice
;
pro thm_part_slice2d_type1, datapoints, velocities, resolution, slice_orient, $
                            part_slice=part_slice, $
                            xgrid=xgrid, ygrid=ygrid, $
                            slice_width=slice_width, $
                            shift=shift, $
                            fail=fail

    compile_opt idl2, hidden


  ; Get slice orienation vectors
  thm_part_slice2d_orientslice, slice_orient, fail=fail, $
                                displacement = displacement, $
                                norm = norm, xvec = xvec
  if keyword_set(fail) then return

  ; Center of distribution
  center = ( [0,0,1] * displacement)

  ; Create square grid
  velmm = [ [minmax(velocities[*,0])], [minmax(velocities[*,1])], [minmax(velocities[*,1])] ]
  xgrid = interpol(velmm[*,0], resolution)
  ygrid = interpol(velmm[*,1], resolution)

  ; Extract slice from scattered points 
  part_slice = thm_part_slice_extract(datapoints, velocities, resolution, $
                                    center, norm, xvec, slice_width=slice_width, $
                                    xgrid=xgrid, ygrid=ygrid, shift=shift, $
                                    fail=fail)
  if keyword_set(fail) then return
  
end



;+
;Procedure: thm_part_slice2d
;
;Purpose: Returns a 2-D slice of the 3-D THEMIS ESA/SST ion or electron distribution
;         function.  The 2-D slice is returned via the PART_SLICE, XGRID, YGRID, and 
;         SLICE_INFO keywords.
;         
;         This procedure works in conjunction with thm_part_dist_array.pro and 
;         plot_part_slice2d.pro.  The corresponding crib is thm_crib_part_slice2d.pro 
;
;   There are currently three methods of generating slices for plotting:
;
;   2D Nearest Neighbor (default):
;     A slice of data with user defined width is projected onto the slice plane 
;     and interpolated onto a regular grid using the nearest neighbor method.  
;
;   2D Interpolation (from thm_esa_slice2d):
;     Datapoints within the specified theta or z-axis range are projected onto 
;     the slice plane and linearly interpolated onto a regular 2d grid.
;
;   3D Interpolation:
;     The entire 3-dimensional distribution is linearly interpolated onto a 
;     regular 3d grid and a this slice is extracted.  The slice will consist of
;     up to resolution^2 datapoints from within 0.866/resolution of the total 
;     velocity range along the normal. This method works best with dense data.
;       Note: - Not all points within the aforemention range will be used.
;             - Interpolation may occure across data gaps or areas with recorded zeroes
;             - Higher resolution regridding will severely slow this method
;    
;     
;    *X,Y-range values determined slightly differently for 2D Interpolation
;
;
;Arguments:
; DATARRAY: An array of data structures as returned by one of the get_th?_p???
;            routines (or thm_part_dist_array.pro).  The structure must have
;            a magf field containing a three-component magnetic field vector.
;
;Input Keywords:
; SLICE_TIME: Beginning of time window in seconds since Jan. 1, 1970.  If
;             CENTER_TIME keyword set, then TIME is the center of the time widow
;             specified by the TIMEWIN keyword.
; CENTER_TIME: Flag that, when set, centers the time window around the time 
;              specified by SLICE_TIME keyword.
; TIMEWIN: Length in seconds over which to compute the slice.
; COORD: A string designating what coordinate system the slice will be based off.
;        Default is 'DSL', but can also be 'GSE' or 'GSM'
; SLICE_ORIENT: (ignored for 2D interpolation) 
;               [x, y, z, vel] A four-element array that specifies the
;               orientation of the slice plane.  The first three elements
;               specify the direction of the plane's normal vector.  The fourth
;               element is the distance of the slice plane's center from the
;               origin along the plane's normal vector in units of meters/sec.
;               Default = [0,0,1,0]
; UNITS: A string designating the desired units ('Counts','DF','Rate','CRate',
;        'Flux','EFlux','E2Flux','E3Flux') - (DF by default)
; RESOLUTION: A single integer specfying the resolution of each dimension of the
;             slice (as well as the interpolated volume in the the 3d case).
; REGRID: A three element array specifying regrid dimensions in phi, theta, and 
;         energy respectively. If set, all distributions' data will first be 
;         spherically interpolated using the nearest neighbor method. For example  
;         the default [32,16,128] will yield distributions with 32x16 points per 
;         energy, and 128 energy bins). Increasing the regridding resolution will
;         slow slice production, particularly if using 3D interpolation. 
; ROTATION: suggesting the x and y axis, which can be specified as the following:
;   'BV': The x axis is parallel to B field; the bulk velocity defines the x-y plane. (DEFAULT)
;   'BE': The x axis is parallel to B field; the B x V(bulk) vector defines the x-y plane.
;   'xy'/'xyz': The x axis is V_x and the y axis is V_y.
;   'xz': The x axis is V_x and the y axis is V_z.
;   'yz': The x axis is V_y and the y axis is V_z.
;   'xvel': The x axis is V_x; the y axis is defined by the bulk velocity. 
;   'perp': The x axis is the bulk velocity projected onto the plane normal to the B field; y is B x V(bulk)
;   'perp_xy'/'perp_xyz': Geometric xy coordinates projected onto the plane normal to the B field.
;   'perp_xz': Geometric xz coordinates projected onto the plane normal to the B field.
;   'perp_yz': Geometric yz coordinates projected onto the plane normal to the B field.
; ERANGE: Two element array specifying the energy range to be used
; SUBTRACT: Flag to subtract the bulk velocity before plot.  The exact quantity 
;           to use can be specified in the call to thm_part_dist_array.
; SLICE_WIDTH: (2d nearest neighbor only)
;              Specifies width of the slice in % of the total range
; SMOOTH: An odd integer >=3 specifing the width of the smoothing window in # 
;         of points. Even entries will be incrimented, 0 and 1 are ignored.
; THETARANGE: (2D interpolation only)
;             Angle range, in degrees [-90,90], used to calculate slice.
;             Default is [-20,20], will override ZDIRRANGE. 
; ZDIRRANGE: (2D interpolation only)
;            Z-Axis range, in km/s, used to calculate slice.
;            Ignored if called with THETARANGE.
;
;Output variables for plotting:
; PART_SLICE: The 2-D data array to be plotted (the slice).
; XGRID: Array of x-locations for the slice.
; YGRID: Array of y-locations for the slice.
; SLICE_INFO: Structure to be passed to plot_part_slice2d to determine plot labels.
;      {
;       probe: string containing the probe
;       dist: string containing the type of distribution used
;       coord: string describing the coordinate system used for the slice
;       rot: string describing the user specified rotation (N/A for 2D interp)
;       units: string describing the units
;       twin: string describing the time window of the slice
;       range: array containing the range of the un-interpolated data 
;       erange: array containing the energy range used
;       trange: array countaining the numerical time range
;       ndists: number of distributions used
;       qmass: factor for plotting [(J/eV) / particle mass]
;       coord_m: Rotation matrix from original data's coordinates (DSL) to
;                those specified by the COORD keyword.
;       rot_m: Rotation matrix from the the specified coordinates to those 
;            defined by ROTATION (column matrix of new coord's basis).
;            (N/A for 2D interp)
;       orient_m: Rotation matrix from the coordinates defined by ROTATION to 
;                 the arbitrary coordinates defined by SLICE_ORIENT 
;                 (column matrix of new coord's basis).
;                 (N/A for 2D interp)
;       }
;
;CAVEATS: Due to IDL software constraints areas of no data are assigned zeros
;         instead of NaNs. This yields a degeneracy between data gaps and 
;         measured zeros for both plots and numerical data.
;         (Such degeneracy would always exist in any logarithmic plot made
;         with the contour procedure)
;
;CREATED BY: Bryan Kerr based on Arjun Raj original thm_esa_slice2d.pro
;            Modified by A. Flores
;
;EXAMPLES:  see the crib file: thm_crib_part_slice2d.pro
;-

pro thm_part_slice2d, datArray, xvels=xvels, yvels=yvels, zvels=zvels, $
                      datapoints=datapoints, slice_time=slice_time, $
                      center_time=center_time, timewin=timewin, $
                      slice_orient=slice_orient, resolution=resolution, $
                      part_slice=part_slice, xgrid=xgrid, ygrid=ygrid, zgrid=zgrid, $
                      rotation=rotation, erange=erange, coord=coord, $
                      subtract=subtract, type=type, units=units, $
                      thetarange=thetarange, zdirrange=zdirrange, $
                      smooth=smooth, regrid=regrid, slice_width=slice_width, $
                      ; passing back to calling program for plotting purposes
                      slice_info=slice_info, $
                      _EXTRA = e, fail=fail

if size(datArray,/type) ne 8 then begin
  fail = 'Invalid data structure.  Input must be valid array of '+ $
          'ESA or SST distributions with required tags. '+ $
          'See thm_part_dist_array.'
  dprint, fail
  return
endif

if ~keyword_set(slice_time) then begin
  fail = 'Please specifiy a time at which to compute the slice.'
  dprint, fail
  return
endif

if ~keyword_set(timewin) then begin
  fail = 'Please specifiy a time window for the slice."
  dprint, fail
  return
endif


; Defaults
;
fail = ''
q = 1.602176e-19 ; J/eV
conv = 6.2508206e24 ; convert distrubution mass to kg
if ~keyword_set(coord) then coord='DSL'
if ~keyword_set(rotation) then rotation='BV'
if ~keyword_set(units) then units = 'df'
if ~keyword_set(resolution) then resolution = 150 else resolution = float(resolution)
if size(regrid,/type) eq 0 then regrid=['64','32','64'] else regrid = float(regrid)
if size(type,/type) eq 0 then type = 1
if type eq 2 then begin
  if ~keyword_set(thetarange) and ~keyword_set(zdirrange) then begin
    thetarange = [-20,20]
  endif
endif else begin
  if type ne 3 then type = 1
  if ~keyword_set(slice_orient) then slice_orient = [0,0,1,0]
  if rotation eq 'xyz' then rotation = 'xy'
  if rotation eq 'perp_xyz' then rotation = 'perp_xy'
endelse
stype = (['2D nn','2D i','3D i'])[type-1] ; for plot labels
probe = keyword_set(datArray[0].spacecraft) ? datArray[0].spacecraft : $
                 strmid(datArray[0].project_name, 0, /reverse_offset)
if ~stregex(probe, '[abcde]', /bool, /fold_case) then probe = ''



; Get time info
;
; get center times of each dat structure
times = datArray.time + (datArray.end_time - datArray.time)/2

; get the boundaries of the time window
if keyword_set(center_time) then begin
  time_min = slice_time - timewin/2
  time_max = slice_time + timewin/2
endif else begin
  time_min = slice_time
  time_max = slice_time + timewin
endelse

; get indexes of dat structures in reqested time window
times_ind = where(times ge time_min AND times le time_max, ndat)
if ndat gt 0 then begin
  datStructs = datArray[times_ind]
endif else begin
  fail = 'No particle distributions exist in data array that are within the requested time window.' 
  dprint, fail 
  return
endelse



; Loop over distributions
; 
; Convert data into the requested units and get a new set of cartesian velocity 
; bins based off the distributions energy and angle bins.  If regridding 
; then all data will first be interpolated onto a regular grid in spherical 
; coordinates.  The actual data values will be averaged after the loop. 
;-----------------------
for in=0,ndat-1 do begin
  thedata=datStructs[in]
  
  ; Convert to specified units ('df' is default)
  thedata = conv_units(temporary(thedata),strlowcase(units))

  
  ;Verify valid data
  if thedata.valid ne 1 then begin
    fail = 'Not valid data for disribution at: '+time_string(thedata.time)
    dprint, fail
    return
  endif


  ;Verify that the angle bins do not differ from the last distribution's
  ; If there's no change then cartesian bins need only be found once
  if ~thm_part_slice2d_checkbins(thedata, datstructs, fail=fail) then return


  ; Modify requested erange to values of bins in that range
  if not keyword_set(erange) then begin
    erange = [min(thedata.energy), max(thedata.energy)]
    eindex = indgen(thedata.nenergy)
  endif else begin
    eindex = where(thedata.energy[*,0] ge erange[0] and thedata.energy[*,0] le erange[1], nebins)
    if nebins ne 0 then begin 
      erange = [min(thedata.energy[eindex,0]),max(thedata.energy[eindex,0])]
    endif else begin
      fail = 'No energy bins exist in this data set within the specified limits.'
      dprint, fail
      return
    endelse
  endelse


  ; Determine x,y,z velocity components. If re-gridding is set then the distribution 
  ; will first be re-gridded in spherical coordinates. 
  thm_part_slice2d_getXYZ, thedata, in, regrid=regrid, $
                           validbins=validbins, currbins=currbins, $
                           erange=erange, q=q, conv=conv, $
                           fail=fail, skip=skip, $
                           ncounts = ncounts, $
                           xvels = xvels, $
                           yvels = yvels, $
                           zvels = zvels, $
                           orange = orange
  if keyword_set(fail) then return
  if keyword_set(skip) then continue

  ; Create array storing the total number of valid data points
  ; for each velocity.  This will be used to average the date later
  weight = (ncounts gt 0) and (currbins)
  weight_t = keyword_set(weight_t) ? weight_t + weight:weight

  ; Sum of counts at each time sample
  ncounts_t = (in eq 0) ? ncounts:ncounts_t+ncounts
  
endfor
;--------------------------
;End loop over distributions


; Create array containing x,y,z
velocities = fltarr(n_elements(xvels),3,/nozero)
velocities[*,0] = temporary(xvels)
velocities[*,1] = temporary(yvels)
velocities[*,2] = temporary(zvels)


; Remove points that never had any valid data within the specified ranges
valid = where(validbins, nvalid)
if nvalid gt 0 then begin
  ncounts_t = ncounts_t[valid]
  weight_t = weight_t[valid]
  velocities = velocities[valid,*]
endif else begin
  fail = 'No valid data in distribution(s).  This is probably due to stringent energy limits.'
  dprint, fail
  return
endelse


; Average data over number of time samples containging measurements
wz = where(weight_t eq 0, nwz)
if nwz gt 0 then weight_t[wz] = 1b ; don't diving by zero!
datapoints = ncounts_t / weight_t


; For testing only: shows 3-D distribution of nonzero data
;rr = where(datapoints ne 0)
;rg = [-max(velocities),max(velocities)]
;iplot, velocities[rr,0], velocities[rr,1], velocities[rr,2], linestyle=6, sym_index=1, $
;       xrange = rg, yrange=rg, zrange=rg, $
;       rgb_table=13, vert_colors=bytscl(alog10(datapoints[rr]))


; Get mag data
bfield = thm_part_slice2d_getmag(thedata, datstructs, magdata=magdata, rotation=rotation, fail=fail)
if n_elements(bfield) eq 1 then return


; Get bulk velocity (in m/s)
thevel = thm_part_slice2d_getvel(thedata, datstructs, veldata=veldata, rotation=rotation, fail=fail)
if n_elements(thevel) eq 1 then return


; Subtract bulk velocity vector?
if keyword_set(subtract) then begin
  thm_part_slice2d_subtract, velocities, thevel, veldata
endif


; Rotate to GSM or GSE if requested
thm_part_slice2d_cotrans, velocities, thevel, bfield, probe=probe, ct=ct, $
                          coord=coord, twin=[time_min,time_max], fail=fail
if keyword_set(fail) then return


; Rotate data to desired coordinates
thm_part_slice2d_rotate, velocities, rot, bfield, thevel, rotation=rotation, fail=fail
if keyword_set(fail) then return


; Convert velocities to km/s
factor = 1000.
velocities = temporary(velocities)/factor
thevel = thevel/factor


; Apply theta or z-axis limits (primarily for old method, see documentation)
thm_part_slice2d_2dlimits, velocities, datapoints, thetarange=thetarange, zdirrange=zdirrange, fail=fail
if keyword_set(fail) then return


; Remove negative values and warn
index = where(datapoints ge 0,count, ncomplement=ncount)
if ncount ne 0 then begin
  if count ne 0 then begin
    velocities = velocities[index,*]
    datapoints = datapoints[index]
  endif else begin
    fail = 'No non-negative data values in transformed data (something is wrong if you are reading this).'
    dprint, fail
    return
  endelse
endif


; Sort transformed velocity grid
sorted = sort(velocities[*,0])
velocities = velocities[sorted,*]
datapoints = datapoints[sorted]




;*****
; Types of interpolation (see main description):
;
; TYPE=1   2D Nearest Neighbor
; TYPE=2   2D Linear Interpolation
; TYPE=3   3D Linear Interpolation
; 
;*****


; Linear 2D interpolation from thm_esa_slice2D rewritten 
;-------------------------------------------------------
if type eq 2 then begin

  dprint, 'Using 2d linear interpolation'
  thm_part_slice2d_type2, datapoints, velocities, resolution, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail

; 3D Interpolated slices
;------------------------
endif else if type eq 3 then begin

  dprint, 'Using 3d linear interpolation'
  thm_part_slice2d_type3, datapoints, velocities, resolution, $
                          slice_orient, orange, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail

; 2D Nearest Neighbor
; -------------------
endif else begin

  dprint, 'Using 2d nearest neighbor'
  thm_part_slice2d_type1, datapoints, velocities, resolution, slice_orient, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          slice_width=slice_width, $
                          shift=keyword_set(subtract) ? thevel:0, $
                          fail=fail

endelse

if keyword_set(fail) then begin
  dprint, fail
  return
endif


; Apply smoothing
if keyword_set(smooth) then begin
  smooth = round(smooth)
  if smooth ge 2 then begin 
    if smooth mod 2 eq 0 then begin
      dprint, 'No even smoothing values, data will be smoothed over ' + $
              strtrim(smooth,2) + '+1 points.'
    endif else begin
      dprint, 'Smoothing over '+strtrim(smooth,2)+' points.'
    endelse
    part_slice = smooth(part_slice, smooth)
    negidx = where(part_slice lt 0, nneg)
    if nneg gt 0 then begin
;      dprint, 'Negative values created from smoothing. Removing...'
      part_slice[negidx] = 0
    endif
  endif
endif


;Ensure user defined rotations are passed out
if type eq 1 or type eq 3 then begin
  thm_part_slice2d_orientslice, slice_orient, fail=fail, $
                                orient=orient
endif


; Pass out slice information for plotting
slice_info = {probe: probe, $
              dist: thedata.data_name, $
              coord: coord, $
              rot: rotation, $
              units: thedata.units_name, $
              twin: timewin, $
              type: stype, $
              range: orange, $
              erange: erange, $
              trange: [datstructs(0).time, thedata.end_time], $ ;time range of slice data
              ndists: n_elements(datstructs), $ ;ndat ;# distributions used
              qmass: q / (thedata.mass / conv) , $
              shift: keyword_set(subtract) ? thevel:0, $
              coord_m: keyword_set(ct) ? ct:-1, $ ;coordinate transform from DSL
              rot_m: rot, $ ;rotation matrix
              orient_m: keyword_set(orient) ? orient:-1 $ ;slice plane orientation matrix
              }

return

end
