;+
; NAME:
;    SPINMODEL_RESET.PRO
;
; PURPOSE:  
;    Free any data structures associated with the spin model for a 
;    given probe or set of probes, and null out the spin model
;    pointer to prevent stale, previously loaded data from being
;    used erroneously.
;
; CATEGORY: 
;   TDAS
;
; CALLING SEQUENCE:
;   spinmodel_reset,probe='a'
;   spinmodel_reset,probe='a b c'
;   spinmodel_reset,probe=['a','b','c']
;   spinmodel_reset,probe='*'
;   spinmodel_reset,probe='all'
;
;  INPUTS:
;    None.
;
;  OUTPUTS:
;    None.
;
;  KEYWORDS:
;    PROBE  Specifies which probe or probes to reset.
;    
;  PROCEDURE:
;    Parse value of PROBE keyword to see which models to reset;
;    for any specified model with existing data, free the data
;    and set the spinmodel pointer in the common block to null.
;
;  EXAMPLE:
;
;Written by: Jim Lewis (jwl@ssl.berkeley.edu)
;-

; Helper function: free substructures given a spin model pointer

pro spinmodel_freedata_single,mptr
  if ptr_valid(mptr) then begin
     ptr_free,(*mptr).segs_ptr
     ptr_free,(*mptr).spincorr_times
     ptr_free,(*mptr).spincorr_vals
  endif
end

; Helper function: free top-level spinmodel structure for a single probe

pro spinmodel_reset_single,probe=probe
common spinmodel_common,tha_sm_ptr,thb_sm_ptr,thc_sm_ptr,thd_sm_ptr,$
   the_sm_ptr,thf_sm_ptr
case probe of
 'a': BEGIN
      if ptr_valid(tha_sm_ptr) then begin
         spinmodel_freedata_single,tha_sm_ptr
         ptr_free,tha_sm_ptr
         tha_sm_ptr = ptr_new()
      endif
      END
 'b': BEGIN
      if ptr_valid(thb_sm_ptr) then begin
         spinmodel_freedata_single,thb_sm_ptr
         ptr_free,thb_sm_ptr
         thb_sm_ptr = ptr_new()
      endif
      END
 'c': BEGIN
      if ptr_valid(thc_sm_ptr) then begin
         spinmodel_freedata_single,thc_sm_ptr
         ptr_free,thc_sm_ptr
         thc_sm_ptr = ptr_new()
      endif
      END
 'd': BEGIN
      if ptr_valid(thd_sm_ptr) then begin
         spinmodel_freedata_single,thd_sm_ptr
         ptr_free,thd_sm_ptr
         thd_sm_ptr = ptr_new()
      endif
      END
 'e': BEGIN
      if ptr_valid(the_sm_ptr) then begin
         spinmodel_freedata_single,the_sm_ptr
         ptr_free,the_sm_ptr
         the_sm_ptr = ptr_new()
      endif
      END
 'f': BEGIN
      if ptr_valid(thf_sm_ptr) then begin
         spinmodel_freedata_single,thf_sm_ptr
         ptr_free,thf_sm_ptr
         thf_sm_ptr = ptr_new()
      endif
      END
else: message,'Unrecognized probe identifier: '+probe
endcase  
end

pro spinmodel_reset,probe=probe
if (n_elements(probe) EQ 0) then begin
   message,'Required PROBE keyword not passed.'
endif

if (size(probe,/n_dim) EQ 0) then begin
   probe_arr = strsplit(strlowcase(probe),' ',/extract)
endif else begin
   probe_arr = strlowcase(probe)
endelse

; probe_arr is now a lower-case array of probe IDs.

; Check for "all" or "*"

result=WHERE(strmatch(probe_arr,'all'),all_count)
result=WHERE(strmatch(probe_arr,'\*'),star_count)
if ((all_count GE 1) or (star_count GE 1)) then begin
   print,'Resetting spin models for all probes.'
   spinmodel_reset_single,probe='a'
   spinmodel_reset_single,probe='b'
   spinmodel_reset_single,probe='c'
   spinmodel_reset_single,probe='d'
   spinmodel_reset_single,probe='e'
   spinmodel_reset_single,probe='f'
endif else begin
   for i=0,n_elements(probe_arr)-1 do begin
      print,'Resetting spin model for probe '+probe_arr[i]
      spinmodel_reset_single,probe=probe_arr[i]
   endfor
endelse
end
